@@ -7,42 +7,61 @@ backward <- function (p) {
 				p$reml <- x
 				p$fit(p,p$formula)
 			})
-			if (all(sapply(res,conv))) {
+			conv <- lapply(res,converged)
+			if (all(unlist(conv))) {
 				p$cur.reml <- res[[1]]
 				p$cur.ml <- res[[2]]
 				return(p)
 			}
-			p <- reduce.noconv(p)
+			p <- reduce.model(p,conv)
 			if (!any(!is.na(p$tab$block))) return(p)
 		}
 	}
 
-	if (is.null(p$tab)) p$tab <- tabulate.formula(p$formula)
-	if (p$parallel) parallel::clusterExport(p$cl,c('conv','build.formula','unravel','can.remove','get2LL','getdf'),environment())
+	if (is.null(p$tab)) {
+		p$tab <- tabulate.formula(p$formula)
+	}
+	if (!p$crit.name %in% colnames(p$tab)) {
+		if (nrow(p$tab)) {
+			p$tab$Iteration <- 0
+			p$tab[,p$crit.name] <- rep(NA,nrow(p$tab))
+		}
+	}
+	if (p$parallel) {
+		parallel::clusterExport(p$cl,c('converged','build.formula','unravel','can.remove','get2LL','getdf'),environment())
+	}
+
+	iter <- 0
 	repeat {
-		need.reml <- p$can.use.reml && p$reduce.random && any(sapply(unique(p$tab$block),function (b) {
+		need.ml <- !p$force.reml
+		need.reml <- p$force.reml || (p$can.use.reml && any(sapply(unique(p$tab$block),function (b) {
 			i <- which(p$tab$block == b)
 			any(!is.na(p$tab[i,'grouping']))
-		}))
-		if (need.reml && is.null(p$cur.ml) && is.null(p$cur.reml)) p <- fit.references.parallel(p)
-		if (is.null(p$cur.ml)) {
-			progress('Fitting ML reference model')
-			p$reml <- FALSE
-			p$cur.ml <- p$fit(p,p$formula)
-			if (!conv(p$cur.ml)) {
-				p <- reduce.noconv(p)
-				if (!any(!is.na(p$tab$block))) return(p)
-				p <- fit.references.parallel(p)
+		})))
+		if (need.ml && need.reml && is.null(p$cur.ml) && is.null(p$cur.reml)) {
+			p <- fit.references.parallel(p)
+		} else {
+			if (need.ml && is.null(p$cur.ml)) {
+				progress('Fitting ML reference model')
+				p$reml <- FALSE
+				p$cur.ml <- p$fit(p,p$formula)
+				conv <- converged(p$cur.ml)
+				if (!conv) {
+					p <- reduce.model(p,conv)
+					if (!any(!is.na(p$tab$block))) return(p)
+					p <- fit.references.parallel(p)
+				}
 			}
-		}
-		if (need.reml && is.null(p$cur.reml)) {
-			progress('Fitting REML reference model')
-			p$reml <- TRUE
-			p$cur.reml <- p$fit(p,p$formula)
-			if (!conv(p$cur.reml)) {
-				p <- reduce.noconv(p)
-				if (!any(!is.na(p$tab$block))) return(p)
-				p <- fit.references.parallel(p)
+			if (need.reml && is.null(p$cur.reml)) {
+				progress('Fitting REML reference model')
+				p$reml <- TRUE
+				p$cur.reml <- p$fit(p,p$formula)
+				conv <- converged(p$cur.reml)
+				if (!conv) {
+					p <- reduce.model(p,conv)
+					if (!any(!is.na(p$tab$block))) return(p)
+					p <- fit.references.parallel(p)
+				}
 			}
 		}
 
@@ -51,26 +70,33 @@ backward <- function (p) {
 			return(p)
 		}
 		progress('Testing terms')
-		results <- p$parply(unique(p$tab$block[!is.na(p$tab$block)]),function (b) {
+		results <- p$parply(unique(p$tab$block),function (b) {
+			if (is.na(b)) {
+				# cannot remove term because of 'include'
+				return(list(val=rep(NA,sum(is.na(p$tab$block)))))
+			}
 			i <- which(p$tab$block == b)
-			if (!can.remove(p$tab,i) || any(paste(p$tab[i,'term'],p$tab[i,]$grouping) %in% paste(p$include$term,p$include$grouping))) return(list(val=rep(NA,length(i))))
-			p$reml <- all(!is.na(p$tab[i,]$grouping))
-			m.cur <- if (need.reml && p$reml) p$cur.reml else p$cur.ml
+			if (!can.remove(p$tab,i) || any(paste(p$tab[i,'term'],p$tab[i,]$grouping) %in% paste(p$include$term,p$include$grouping))) {
+				# cannot remove term due to marginality
+				return(list(val=rep(NA,length(i))))
+			}
+			if (p$force.reml) {
+				p$reml <- TRUE
+				m.cur <- p$cur.reml
+			} else {
+				p$reml <- p$can.use.reml && all(!is.na(p$tab[i,]$grouping))
+				m.cur <- if (need.reml && p$reml) p$cur.reml else p$cur.ml
+			}
 			f.alt <- build.formula(p$dep,p$tab[-i,],p$env)
 			m.alt <- p$fit(p,f.alt)
-			val <- if (conv(m.alt)) p$crit(p,m.alt,m.cur) else NaN
+			val <- if (converged(m.alt)) p$crit(p,m.alt,m.cur) else NaN
 			val <- rep(val,length(i))
 			list(val=val,model=m.alt)
 		})
 		results <- unlist(sapply(results,`[[`,1))
 		p$tab[,p$crit.name] <- results
-		if (is.null(p$results)) {
-			p$tab$Iteration <- 1
-			p$results <- p$tab
-		} else {
-			p$tab$Iteration <- p$results$Iteration[nrow(p$results)] + 1
-			p$results <- rbind(p$results,p$tab)
-		}
+		p$tab$Iteration <- iter <- iter+1
+		p$results <- rbind(p$results,p$tab)
 		progrep <- p$tab
 		progrep$index <- progrep$code <- progrep$ok <- NULL
 		if (p$crit.name %in% c('LRT','LRT2')) progrep[,p$crit.name] <- exp(results)
@@ -84,14 +110,14 @@ backward <- function (p) {
 			return(p)
 		}
 
-		# 4. Remove the worst offender(s) and continue
+		# Remove the worst offender(s) and continue
 		remove <- remove[p$tab[remove,p$crit.name] == max(p$tab[remove,p$crit.name])]
 		p$tab <- p$tab[-remove,]
 		p$formula <- build.formula(p$dep,p$tab,p$env)
 		p$cur.ml <- p$cur.reml <- NULL
 		if (length(results) == 1) {
 			# Recycle the current model as the next reference model
-			p[[ifelse(p$reml,'cur.reml','cur.ml')]] <- results[[remove]]$model
+			p[[if (p$reml) 'cur.reml' else 'cur.ml']] <- results[[remove]]$model
 		}
 		progress('Updating formula: ',p$formula)
 	}
@@ -211,8 +237,9 @@ order <- function (p) {
 		repeat {
 			have <- p$tab
 			cur <- p$fit(p,build.formula(p$dep,have,p$env))
-			if (conv(cur)) break
-			p <- reduce.noconv(p)
+			conv <- converged(cur)
+			if (conv) break
+			p <- reduce.model(p,conv)
 			if (!any(!is.na(p$tab$block))) return(p)
 		}
 		repeat {
@@ -240,10 +267,12 @@ order <- function (p) {
 				rep(mod,nrow(check))
 			})
 			mods <- unlist(mods,recursive=FALSE)
-			check$score <- sapply(mods,function (mod) if (conv(mod)) p$crit(p,cur,mod) else NaN)
+			check$score <- sapply(mods,function (mod) if (converged(mod)) p$crit(p,cur,mod) else NaN)
 			ok <- Filter(function (x) !is.na(x) & !is.nan(x),check$score)
 			if (!length(ok)) {
-				progress('None of the models converged - giving up ordering attempt.')
+				statuses <- sapply(mods,function (mod) attr(converged(mod),'reason'))
+				statuses <- statuses[!is.na(statuses)]
+				progress('None of the models converged - giving up ordering attempt. The types of convergence failure are:\n',paste(unique(statuses),collapse='\n    '))
 				p$tab <- have
 				p$model <- cur
 				return(p)
@@ -257,8 +286,9 @@ order <- function (p) {
 				# needs an extra fit to obtain the new 'current' model.
 				form <- build.formula(p$dep,have,p$env)
 				cur <- p$fit(p,form)
-				if (!conv(cur)) {
-					progress('The reference model for the next step failed to converge - giving up ordering attempt.')
+				conv <- conv(cur)
+				if (!conv) {
+					progress('The reference model for the next step failed to convergederge - giving up ordering attempt.\nThe failure was: ',attr(conv,'reason'))
 					return(p)
 				}
 			}
@@ -279,18 +309,23 @@ order <- function (p) {
 	else p$tab <- cbind(tab[0,],ok=logical(),score=numeric())
 	if (!is.null(p$include)) p$tab <- rbind(p$tab,transform(p$include,block=NA,ok=TRUE,score=NA))
 
-	p$reml <- FALSE
+	p$reml <- p$force.reml
 	if (any( fxd)) p <- reorder(p,tab[fxd,])
 	if (any(!fxd)) {
-		p$reml <- TRUE
+		p$reml <- p$can.use.reml
 		p <- reorder(p,tab[!fxd,])
 	}
 	p$formula <- build.formula(p$dep,p$tab,p$env)
 	p
 }
 
-reduce.noconv <- function (p) {
-	progress('Convergence failure. Reducing terms and retrying...')
+reduce.model <- function (p,conv) {
+	if (length(conv) == 1) {
+		progress('Convergence failure. Reducing terms and retrying...\nThe failure was: ',attr(conv,'reason'))
+	} else {
+		statuses <- sapply(conv,function (x) attr(x,'reason'))
+		progress('Convergence failure. Reducing terms and retrying...\nThe failures were: ',paste(unique(statuses),collapse='\n    '))
+	}
 	cands <- p$tab$block[!is.na(p$tab$block)]
 	if (length(unique(cands)) < 2) {
 		stop('No terms left for reduction, giving up')
